@@ -182,18 +182,17 @@ def cacheable(fn):
     # type: (T) -> T
     @proxies(fn)
     def cached_strategy(*args, **kwargs):
-        kwargs_cache_key = set()
         try:
-            for k, v in kwargs.items():
-                kwargs_cache_key.add((k, convert_value(v)))
+            kwargs_cache_key = {(k, convert_value(v)) for k, v in kwargs.items()}
         except TypeError:
             return fn(*args, **kwargs)
         cache_key = (fn, tuple(map(convert_value, args)), frozenset(kwargs_cache_key))
         try:
-            return STRATEGY_CACHE[cache_key]
+            if cache_key in STRATEGY_CACHE:
+                return STRATEGY_CACHE[cache_key]
         except TypeError:
             return fn(*args, **kwargs)
-        except KeyError:
+        else:
             result = fn(*args, **kwargs)
             if not isinstance(result, SearchStrategy) or result.is_cacheable:
                 STRATEGY_CACHE[cache_key] = result
@@ -1236,7 +1235,7 @@ def builds(
             "builds(), but is only allowed as a keyword arg"
         )
     required = required_args(target, args, kwargs) or set()
-    to_infer = set(k for k, v in kwargs.items() if v is infer)
+    to_infer = {k for k, v in kwargs.items() if v is infer}
     if required or to_infer:
         if isclass(target) and attr.has(target):
             # Use our custom introspection for attrs classes
@@ -1305,6 +1304,25 @@ def from_type(thing):
        other elements in the lookup.
     4. Finally, if ``thing`` has type annotations for all required arguments,
        it is resolved via :func:`~hypothesis.strategies.builds`.
+
+    There is a valuable recipe for leveraging ``from_type()`` to generate
+    "everything except" values from a specified type. I.e.
+
+    .. code-block:: python
+
+        def everything_except(excluded_types):
+            return (
+                from_type(type).flatmap(from_type)
+                .filter(lambda x: not isinstance(x, excluded_types))
+            )
+
+    For example, ``everything_except(int)`` returns a strategy that can
+    generate anything that ``from_type()`` can ever generate, except for
+    instances of :class:python:int, and excluding instances of types
+    added via :func:~hypothesis.strategies.register_type_strategy.
+
+    This is useful when writing tests which check that invalid input is
+    rejected in a certain way.
     """
     # TODO: We would like to move this to the top level, but pending some major
     # refactoring it's hard to do without creating circular imports.
@@ -2246,3 +2264,42 @@ def functions(like=lambda: None, returns=none()):
             "but got non-callable like=%r" % (nicerepr(like),)
         )
     return FunctionStrategy(like, returns)
+
+
+@composite
+def slices(draw, size):
+    """Generates slices that will select indices up to the supplied size
+
+    Generated slices will have start and stop indices that range from 0 to size - 1
+    and will step in the appropriate direction. Slices should only produce an empty selection
+    if the start and end are the same.
+
+    Examples from this strategy shrink toward 0 and smaller values
+    """
+    check_valid_integer(size)
+    if size is None or size < 1:
+        raise InvalidArgument("size=%r must be at least one" % size)
+
+    min_start = min_stop = 0
+    max_start = max_stop = size
+    min_step = 1
+    # For slices start is inclusive and stop is exclusive
+    start = draw(integers(min_start, max_start) | none())
+    stop = draw(integers(min_stop, max_stop) | none())
+
+    # Limit step size to be reasonable
+    if start is None and stop is None:
+        max_step = size
+    elif start is None:
+        max_step = stop
+    elif stop is None:
+        max_step = start
+    else:
+        max_step = abs(start - stop)
+
+    step = draw(integers(min_step, max_step or 1))
+
+    if (stop or 0) < (start or 0):
+        step *= -1
+
+    return slice(start, stop, step)
